<?php

/**
 * ImxToHelper.php
 *
 * @version $Id: $
 * @package Ripper
 * @copyright 2019
 */

/**
 * Class ImxToHelper
 *
 * the listing page has links like:
 * <a href="https://imx.to/img-5ac47f977c626.html" target="_blank"><img src="https://imx.to/upload/small/2018/04/04/5ac47f977c5e9.jpg" border="0" alt="" /></a>
 *
 * the detail page has this HTML that loads the image:
 * <div style="text-align:center;color:#5c5c5c;font-size:20px;margin-bottom:20px;">100799-H-039</div><div style="text-align:center;"><a href="https://imx.to/u/i/2018/04/04/5ac4802060f51.jpg" title="100799-H-039" target="_blank"><img class="centred" src="https://imx.to/u/i/2018/04/04/5ac4802060f51.jpg" alt="100799-H-039" style="display:inline;max-width:1000px;cursor: -moz-zoom-in;cursor: -webkit-zoom-in;cursor: zoom-in;" /></a></div><div style='text-align:center;color:#5c5c5c;font-size:10px;margin-top:20px;'>[ FILESIZE <span style='color:#8C8C8C;'>2.29 MB</span> ] - [ DIMENSIONS <span style='color:#8C8C8C;'>6048x4032 px</span> ] - [ HASH <span style='color:#8C8C8C;'>ff6482c089a34a0b8e357278cd6a984d</span> ]</div><div style="text-align:center;width:1000px;margin-top:25px;">
 * 
 * in this case the image is: 
 * https://imx.to/u/i/2018/04/04/5ac4802060f51.jpg
 * 
 * https://imx.to/u/t/2019/09/21/2575pu.jpg
 * https://imx.to/u/i/2019/09/21/2575pu.jpg
 */
class ImxToHelper extends BaseHelper
{
    /**
     * for Sept 30 debugging
     * https://imx.to/i/2575ps
     * points to page with form
     * when posted gets to a detail page with
     * https://imx.to/u/i/2019/09/21/2575ps.jpg
     */

    /**
     * for Oct 10 debugging
     * https://imx.to/img-599f69945d510.html
     * points to a page with a form 'Continue to image ...'
     */

    /**
     * process
     * looks like uri can either be:
     * https://imx.to/i/2575ps
     * or
     * https://imx.to/img-599f698f41be7.html
     * for the second version I need to post the form
     *
     * @param $uri
     * @return array Return array with 'image' and 'name' as the keys
     */
    public function process($uri) {
        verbose($uri, $this->verbose);
        $pos = strpos($uri, 'https://imx.to/img');
        if ($pos !== false) {
            $href = $this->extractFormUri($uri);
            verbose('we have to post the Continue to image form', $this->verbose);
            $page = $this->postForm($href);
//          $page = file_get_contents('test/detail.imx.to.html');
            $info = $this->getImageInfo($page);
        } else {
            verbose('we can derive the URI of the image from the thumbnail URI', $this->verbose);
            $info = $this->getImageInfo($uri);
        }

        $image['name'] = $info['name'];
        $image['image'] = $this->getPage($info['link']);
        return $image;
    }

    /**
     * going to receive something like:
     * <a href="https://imx.to/img-599f698f41be7.html" target="_blank"><img src="https://imx.to/upload/small/2017/08/25/599f698f41baa.jpg" border="0" alt="" /></a>
     * just need the href
     * @param $html
     */
    public function extractFormUri($html) {
        $rip = new Ripper;
        $uri = $rip->extractUri($html);
        return $uri;
    }

    public function postForm($uri) {
        $uri = str_replace('http:', 'https:', $uri);
        $opts = array(
            'CURLOPT_POSTFIELDS'=>"imgContinue=Continue+to+image+...+",
            'CURLOPT_REFERER'=>$uri,
            //'CURLOPT_HEADER'=>1
        );
        $retval = $this->downloader->postForm($uri, $opts);
        return $retval;
    }

    /**
     * looking for this img HTML
     * <img class="centred" src="https://imx.to/u/i/2018/04/04/5ac4802060f51.jpg" alt="100799-H-039" style="display:inline;max-width:1000px;cursor: -moz-zoom-in;cursor: -webkit-zoom-in;cursor: zoom-in;" />
     * or maybe looking in the url to the page with this HTML
     * <a href="https://imx.to/i/2575pu" target="_blank"><img src="https://imx.to/u/t/2019/09/21/2575pu.jpg" border="0" alt="" /></a>
     * @param $page
     * @return bool
     */
    public function getImageInfo($page) {
        $retval = false;
        $images = parse_array($page, '<img', '>');
        
        foreach ($images as $image) {
            if (strstr($image, 'https://imx.to/u/')) {
                $src = $this->getAttribute($image, 'src');
                $retval['name'] = basename($src);
                $search = '/t/';
                $replace = '/i/';
                $retval['link'] = str_replace($search, $replace, $src);
                // $retval['name'] = $this->getAttribute($image, 'alt');
                // $retval['link'] = $this->getAttribute($image, 'src');
            }
        }
        return $retval;
    }
    
    public function sendFullSnipet() {
        return true;
    }
}