<?php

/**
 * Ripper.php
 *
 * @version $Id: $
 * @package Ripper
 * @copyright 2019
 */
class Ripper
{
    /**
     * helper is the class that will load the images from the image host
     * @var
     */
    public $helper;

    /**
     * destination folder for the images, where the images get saved
     * @var $dest_folder
     */
    protected $dest_folder;
    
    protected $verbose = false;
    
    protected $start = 0;

    protected $config;

    protected $downloader;

    protected $progress_counter = 0;
    protected $progress_interval = 10;
    protected $progress_indicator = '.';
    protected $installed_version = '';
    
    /**
     * fails is an array that holds all the failed image requests
     * @var fails
     */
    protected $fails = array();

    public function __construct()
    {}

    /**
     * load config information from config/config.php
     */
    public function config($conf) {
        if (!array_key_exists('user_agent', $conf)) {
            $conf['user_agent'] = setHttpUserAgent();
        }

        foreach ($conf as $key => $value) {
            $this->config[$key] = $value;
        }

        if (!array_key_exists('application_path', $conf)) {
            $application_path = '/';
        }
        define('APPLICATION_PATH', '/');

        //out($this->config);
    }

    public function getConfigValue($key) {
        if (array_key_exists($key, $this->config)) {
            return $this->config[$key];
        }
        return false;
    }

    /**
     * going to receive something like 'images/MetArt_Jenya-Simivis
     * and need to preserve the 'images/' part
     */
    public function setDestFolder($folder) {
        if (is_null($folder)) {
            return null;
        }
        $pos = strpos($folder, DIRECTORY_SEPARATOR, 6);
        if ($pos) {
            $temp = str_replace(DIRECTORY_SEPARATOR, '-', $folder);
            $folder = str_replace('images-', 'images/', $temp);
        }
        $this->dest_folder = trim($folder);
        return $folder;
    }
    
    /**
     * process
     * we receive a string of HTML that contains URIs for the images, like:
     * <a href="http://img.yt/img-57cdd5df5f9de.html" target="_blank"><img src="http://img.yt/upload/small/2016/09/05/57cdd5df5f9a6.jpg" border="0" alt="" /></a>
     *
     * @param $links
     * @todo This method needs to be reworked.  If there's a list of images with different hosts and one or the hosts
     * does (or doesn't) require the sendImages() functionality, then the rest of the hosts will be treated incorrectly
     */
    public function process($links) {
        if (!is_array($links)) {
            $list = $this->convertLinks($links);
        } else {
            $list = $links;
        }
//        var_dump($list);
        $list = $this->ignoreLink($list);
//        var_dump($list);
        if (($this->start) > 0 ) {
            $list = array_slice($list, $this->start);
        }
//        var_dump($list); exit;
        verbose('found '. count($list) . ' images to process', $this->verbose);

        $this->helper = $this->getHelper($this->extractUri($list[0]));
        $this->helper->setVerbose($this->verbose);
//        $this->helper->setDownloader($this->downloader);

        if(!$this->helper) {
            var_dump($list);
            exit('did not find helper');
        }
        verbose('using '. get_class($this->helper), $this->verbose);

        if ($this->helper->sendImages()) {
            $links = parse_array($links, '<img', '/>');


            $this->helper->setSnippet($links[0]);

            foreach ($links as $uri) {
                $image = $this->helper->process($uri);
                if ($image) {
                    $rs = $this->saveImage($image);
                    if ($rs) {
                        $this->updateProgressCounter();
                        verbose('saved: '. $image['name'], $this->verbose);
                    } else {
                        verbose('failed: '. $image['name'], $this->verbose);
                    }
                }
                $this->displayProgressCounter();
            }
        } else {
            $this->helper->setSnippet($list[0]);

            foreach ($list as $uri) {
                $this->helper = $this->getHelper($this->extractUri($uri));
                if ($this->helper->sendFullSnipet()) {
                    $image = $this->helper->process($uri);
                } else {
                    $image = $this->helper->process($this->extractUri($uri));
                }

                if ($image) {
                    $rs = $this->saveImage($image);
                    if ($rs) {
                        $this->updateProgressCounter();
                        verbose('saved: '. $image['name'], $this->verbose);
                    } else {
                        verbose('failed: '. $image['name'], $this->verbose);
                    }
                } else {
                    verbose('did not find image: '. $this->helper->getMessage() .' using '. get_class($this->helper), $this->verbose);
                }
                $this->displayProgressCounter();
            }
        }
        if (empty($this->fails)) {
            return true;
        }
        return false;
    }

    /**
     * processSet
     */
    public function processSet($set) {
        $count = count($set['links']);
        if ($count == 0) {
            return false;
        }
        $this->helper = $this->getHelper($this->extractUri($set['links'][0]));
        $this->helper->setVerbose($this->verbose);

        foreach ($set['links'] as $link) {
            if ($this->helper->sendFullSnipet()) {
                $image = $this->helper->process($link);
            } else {
                $image = $this->helper->process($this->extractUri($link));
            }
            if ($image) {
                $rs = $this->saveImage($image);
                if ($rs) {
                    $this->updateProgressCounter();
                    verbose('saved: '. $image['name'], $this->verbose);
                } else {
                    verbose('failed: '. $image['name'], $this->verbose);
                }

                //exit;
            }
            $this->displayProgressCounter();
        }
        if (empty($this->fails)) {
            return true;
        }
        return false;
    }

    /**
     * @param $links string An HTML string of hrefs
     * @return array of the links
     */
    public function convertLinks($links)
    {
        $array = parse_array($links, '<a', '</a>');
        return $array;
    }

    public function extractUri($fragment) {
        $start = 'href="';
        $end = '" target';
        $uri = return_between($fragment, $start, $end, EXCL);
        verbose('extracted this URI: ' . $uri, $this->verbose);
        return $uri;
    }

    /**
     * ignoreLink
     * looks in array of links, if item doesn't include img src, then skip it.
     */
    public function ignoreLink($links)
    {
        $retval = false;
        $ignore = 'img src';
        foreach ($links as $link) {
            $pos = strpos($link, $ignore);
            if ($pos === FALSE) {
                continue;
            }
            $retval[] = $link;
        }
        return $retval;
    }
    
    /**
     * http://imgbox.com/lAkZ4Mwd
     * http://img.yt/img-57cdd5df5f9de.html
     * http://www.turboimagehost.com/p/31486507/sb_samira_enticement_2014-12-15_118.jpg.html
     *
     * @param $uri
     */
    public function getHelper($uri) {
        $classname = $this->getClassname($uri);
        if(is_a($this->helper, $classname)) {
            return $this->helper;
        }
        $filename = 'app/Helpers/'.$classname .'.php';
        if (!file_exists($filename)) {
            $filename = 'app/dummyHost.php';
            $classname = 'DummyHost';
        }
        include_once $filename;
        $helper = new $classname;
        $helper->setDownloader($this->downloader);
        return $helper;
    }

    /**
     * Receive something like:
     * http://coreimg.net/uhiw6m1lrj4s
     * or
     * https://i.imgbox.com/YtnopzXN.jpg?download=true
     * And return the correct helper class name
     * @param string $uri
     * @return string
     */
    public function getClassname($uri) {
        $host = parse_url($uri ,PHP_URL_HOST);
        $count = substr_count($host, '.');
        if ($count > 1) {
            $pos = strpos($host, '.');
            $host = substr($host, $pos + 1);
        }
        $temp = str_replace('.', ' ', $host);
        $retval = str_replace(' ', '', ucwords($temp));
        return $retval .'Helper';
    }

    /**
     * @param $image array An array with 'name' and 'image' as the keys
     */
    public function saveImage($image) {
        $rs = false;
        if (!empty($image['image']))
        {
            $name = $this->sanitizeName($image['name']);
            $rs = file_put_contents($this->dest_folder .'/'. $name, $image['image']);
        } else {
            $this->fails[] = $image;
        }
        if ($rs) {
            return true;
        }
        return false;
    }

    public function getFails() {
        return $this->fails;
    }

    public function sanitizeName($name) {
        $name = preg_replace("/\s\s+/", ' ', $name);
        $rs = preg_replace("/[^a-zA-Z0-9\-\.]/", '_', $name);
        $pos = strpos($rs, '.jpg.jpg');
        if ($pos !== false) {
            $rs = str_replace('.jpg.jpg', '.jpg', $rs);
        }
        return $rs;
    }

    public function setVerbose($verbose = false) {
        if ($verbose) {
            $this->verbose = true;
        } else {
            $this->verbose = false;
        }
    }
    
    public function getVerbose() {
        return $this->verbose;
    }

    public function setDownloader(RipperDownloadInterface $downloader) {
        $downloader->init(['user_agent'=>$this->config['user_agent']]);
        $this->downloader = $downloader;
    }

    public function getDownloader() {
        return $this->downloader;
    }

    public function setStart($start = 0) {
        $this->start = $start;
    }
    
    public function processImgur($set) {
        if (($this->start) > 0 ) {
            $set = array_slice($set, $this->start);
        }
        
        foreach ($set as $item) {
            $image = $this->helper->process($item);
            if ($image) {
                $rs = $this->saveImage($image);
                if ($rs) {
                    verbose('saved: '. $image['name'], $this->verbose);
                } else {
                    verbose('failed: '. $image['name'], $this->verbose);
                }
                //exit;
            }
        }
        if (empty($this->fails)) {
            return true;
        }
        return false;
    }

    public function updateProgressCounter() {
        $this->progress_counter++;
    }

    public function displayProgressCounter() {
        $mod = $this->progress_counter % $this->progress_interval;
        if ($mod === 0) {
            echo $this->progress_indicator;
        }
    }
    
    public function getInstalledVersion() {
        if (empty($this->installed_version)) {
            $this->installed_version = file_get_contents('./version.txt');
        }
        return $this->installed_version;
    }
}