<?php
/**
 * RipperDownload.php
 *
 * @version $Id: $
 * @package Ripper
 * @copyright 2019
 */

class RipperDownload implements RipperDownloadInterface
{
    protected $verbose;
    protected $user_agent;
    protected $referrer = null;

    public function __construct($verbose)
    {
        $this->verbose = (bool)$verbose;
    }

    public function init($values) {
        foreach ($values as $key=>$value) {
            $this->$key = $value;
        }
    }

    public function getPage($uri, $opts = null)
    {
        verbose('getPage - looking for: '. $uri, $this->verbose);
        if(empty($this->user_agent)) {
            $this->user_agent = setHttpUserAgent();
        }
        verbose('using user_agent: '. $this->user_agent, $this->verbose);
        $arrContextOptions=array(
            "ssl"=>array(
                "allow_self_signed" => true,
                "verify_peer"=>false,
                "verify_peer_name"=>false,
            ),
            "http" => array(
                "user_agent" => $this->user_agent
            )
        );
        if (empty($uri)) {
            return false;
        }
        $page = file_get_contents($uri, false, stream_context_create($arrContextOptions));
        return $page;
    }

    public function getPageHtml($uri, $opts = null)
    {
        verbose('getPageHtml - looking for: '. $uri, $this->verbose);
        $ch = curl_init();
        $timeout = 5;
        if (!is_null($this->referrer)) {
            curl_setopt($ch, CURLOPT_REFERER, $this->referrer);
        }
        curl_setopt($ch, CURLOPT_URL, $uri);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
        $data = curl_exec($ch);
        curl_close($ch);
        return $data;
    }

    public function postForm($uri, $opts = null)
    {
//        out($opts);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_URL,$uri);
        foreach ($opts as $key=>$value) {
            switch ($key) {
                case 'CURLOPT_POSTFIELDS':
                    curl_setopt($ch, CURLOPT_POSTFIELDS, $value);
                    break;

                case 'CURLOPT_REFERER':
                    curl_setopt($ch, CURLOPT_REFERER, $value);
                    break;

                case 'CURLOPT_HEADER':
                    curl_setopt($ch, CURLOPT_HEADER, $value);
                    break;

                default:
                    curl_setopt($ch, intval($key), $value);
                    break;
            }
        }
//out(curl_getinfo($ch));

        $server_output = curl_exec ($ch);

        curl_close ($ch);
        return $server_output;
    }

    /**
     * wrapper for File_get_contents()
     */
    public function fgcwrapper() {

    }

}