<?php
/**
 * rip.php
 *
 * @version $Id: $
 * @package Ripper
 * @copyright 2019
 */

if (PHP_OS === 'Darwin') {
    passthru('clear');
}
chdir(dirname(__FILE__));

include 'conf/config.php';
include 'conf/functions.php';
include 'lib/LIB_parse.php';
include 'app/BaseHelper.php';
include 'app/Ripper.php';
include 'app/RipperDownloadInterface.php';
include 'app/RipperDownload.php';
//include 'app/RipperHttpful.php';

$args = getArguments();
$verbose = $args['verbose'];
if ($verbose) {
    $str = 'true';
    $conf['verbose'] = 'true';
} else {
    $str = 'false';
}
verbose('set verbose to: '. $str, $verbose);

$rd = new RipperDownload($verbose);
//$rd = new RipperHttpful($verbose);

$rip = new Ripper();
$rip->config($conf);
$rip->setVerbose($args['verbose']);
$rip->setDownloader($rd);

$version = $rip->getInstalledVersion();

echo 'L A X A E R I P P E R '. $version . PHP_EOL;
echo PHP_EOL;
echo '[1] Rip images from multiple albums on one page';
echo PHP_EOL;
echo '[2] Rip images from one album on a page';
echo PHP_EOL;
echo '[3] Rip images from multiple albums listed in a file';
echo PHP_EOL;
echo '[4] Rip images from a pasted list';
echo PHP_EOL;
//echo '[7] Update';
//echo PHP_EOL;
//echo '[8] Config';
//echo PHP_EOL;
echo '[9] Help information';
echo PHP_EOL;
echo 'Select action: ';
$args['action'] = trim(fgets(STDIN));

/**
 * addn is an array that holds additional information specific to the action selected by the user
 */
$addn = [];

switch ($args['action']) {
    case '1':
        $addn = getArgsForCollection();
        break;
    case '2':
        $addn = getArgsForAlbum();
        break;
    case '3':
        $addn = getArgsForAlbumsInFile();
        break;
    case '4':
        $addn = getArgsForList();
        break;
    case '7':
        break;
    case '8':
        break;
    case '9':
        break;

    default:
        displayError(array('No action selected, or invalid selection'));
        break;
}

$args = array_merge($args, $addn);

$rs = false;

switch ($args['action']) {
    case '1':
        require_once 'app/rip_collection.php';
        $rs = rip_collection($args, $rip);
        break;

    case '2':
        require_once 'app/rip_album.php';
        $rs = rip_album($args, $rip);
        break;

    case '3':
        require_once 'app/rip_albums_in_file.php';
        $rs = rip_albums_in_file($args, $rip);
        break;

    case '4':
        require_once 'app/rip_list.php';
        $rs = rip_list($args, $rip);
        break;

    case '7':
        displayError(array('Update is not ready for usage at this time'));
//        require_once 'app/rip_update.php';
//        $rs = rip_update($args, $rip);
        break;
    
    case '8':
        displayConfig();
        break;
    default:
        displayHelp();
        break;

}

verbose($rs, $verbose);

echo PHP_EOL;
echo '---DONE---'. PHP_EOL;
if (PHP_OS === 'Darwin') {
    passthru('tput bel');
}



function getArgsForCollection() {
    $args['tag'] = 'bq';
    echo PHP_EOL;
    echo 'uri: ';
    $args['uri'] = trim(fgets(STDIN));
    echo PHP_EOL;
    echo 'Prefix: ';
    $args['prefix'] = trim(fgets(STDIN));
    echo PHP_EOL;
    echo 'Title in tag (h2 or [bq]): ';
    $tag = trim(fgets(STDIN));
    if(!empty($tag)) {
        $args['tag'] = $tag;
    }

    $error = array();

    if (!$args['uri']) {
        $error[] = 'must supply a URI to the list of images: -u "http://example.com/path/to/links"';
    }

    if (count($error) > 0) {
        displayError($error);
    }
    return $args;
}

function getArgsForAlbum() {
    echo PHP_EOL;
    echo 'title: ';
    $args['title'] = trim(fgets(STDIN));
    echo 'uri: ';
    $args['uri'] = trim(fgets(STDIN));
    echo 'start after image # ';
    $args['start'] = trim(fgets(STDIN));

    if ($args['start'] === '') {
        $args['start'] = 0;
    }
    $error = array();

    if (!$args['title']) {
        $error[] = 'must supply a title for the list of image, like "XArt Mia Inside"';
    }
    if (!$args['uri']) {
        $error[] = 'must supply a URI to the list of images, like "http://example.com/path/to/album"';
    }

    if (count($error) > 0) {
        displayError($error);
    }
    return $args;
}

function getArgsForAlbumsInFile() {
    echo PHP_EOL;
    echo 'filename: ';
    $args['filename'] = trim(fgets(STDIN));

    $error = array();

    if (!$args['filename']) {
        $error[] = 'must supply a filename for the file that contains the list of albums';
    }

    if (count($error) > 0) {
        displayError($error);
    }
    return $args;
}

function getArgsForList() {
    echo PHP_EOL;
    echo 'title: ';
    $args['title'] = trim(fgets(STDIN));
    echo PHP_EOL;
    echo 'links: ';
    $args['links'] = trim(fgets(STDIN));

    $error = array();

    if (!$args['title']) {
        $error[] = 'must supply a title for the list of images, like "XArt Mia Inside"';
    }
    if (!$args['links']) {
        $error[] = 'must supply a batch of images to process';
    }

    if (count($error) > 0) {
        displayError($error);
    }
    return $args;
}

function displayError($error) {
    echo 'ERROR';
    echo PHP_EOL;
    foreach($error as $item) {
        echo $item .PHP_EOL;
    }
    exit;
}

function displayConfig() {
    echo PHP_EOL;
    echo 'Nothing to do here, config is still being developed';
}

function displayHelp() {
    echo PHP_EOL;
    echo 'Help information for Ripper.'. PHP_EOL;
    echo 'This application will download (or rip) images from various image hosts.  Mostly it is intended to be used on https://vipergirls.to'. PHP_EOL;
    echo PHP_EOL;
    echo 'Usage:'. PHP_EOL;
    echo 'Select one of the variety of ways to download images.'.PHP_EOL;
    echo 'Option 1: use this to download all the images in a collection on a page.  For example, this page: https://vipergirls.to/threads/1111573-Galitsin-Works-A-Z-(1999-2019)/page3 contains a dozen or so albums.  You can set an optional prefix, which will be prepended to the folder name, not the images.  Ripper will try to figure out the title for each set (for this page it can not figure out the title).  The title is typically located within  <H2> or <blockquote> tags.  By default Ripper will look in the <blockquote> for the title.  Some Viper users use the <h2> tag, some are all over the place.'.PHP_EOL;
    echo 'Option 2: use this to download all the images in a single album on a page.  For example, this page: https://vipergirls.to/threads/4444467-Martina-Mink-%E2%80%93-Come-On-In-129-pictures-5760px-(18-Apr-2019) has one album.  The album has been posted three different times by three different users.  Ripper will only download the first instance of the album'.PHP_EOL;
    echo 'Option 3: use this to download all albums which are stored in a file.'. PHP_EOL .'The file must be in the root of the application and it must follow the format of either of the example files: example_list.php or example_list.csv.  Albums in the file are treated by LaxaeRipper the same was as Option 2.  You can use either of the example_list files as the basis of your file.  The files must have a title and URI and optionally a \'start\' value.  If a start value is set, we will start the download at that point in the list.  If there are 100 images to download and the start number is 25, then we will skip the first 24 images and start downloading at number 25.'.PHP_EOL;
    echo 'Option 4: use this to download an arbitrary list of images.  Best way to get the list is to view the source of the page and copy the links for the images that you want.  The image links will be between <blockquote class="postcontent restore "> and </blockquote> '.PHP_EOL;
    echo PHP_EOL;
    echo 'Ripper will use the file names supplied by the image host for the images that are being downloaded.  Sometimes those are kind of nasty, like this one: 174ffa1269420444.jpg  At this time I don\'t know how to get a better file name.'. PHP_EOL;
    echo PHP_EOL;
    echo 'Flags:'. PHP_EOL;
    echo '-v sets Ripper to run in verbose mode, which will override whatever is in config'. PHP_EOL;
    echo PHP_EOL;

    $hosts = getSupportedHosts();
    echo PHP_EOL;
    echo 'Supported file hosts: '. PHP_EOL;
    foreach ($hosts as $host) {
        echo $host . PHP_EOL;
    }
}

function getSupportedHosts() {
    //$domains = 'unknown';
    $d = dir("app/Helpers");
    while (false !== ($entry = $d->read())) {
        $ignore = ['.', '..'];
        if (in_array($entry, $ignore)) {
            continue;
        }
        $domain = str_replace('Helper.php', '', $entry);
        $search = ['Cc', 'Com', 'Gy', 'Me', 'Net', 'Org', 'Se', 'Site', 'To', 'Yt'];
        $replace = ['.cc', '.com', '.gy', '.me', '.net', '.org', '.se', '.site', '.to', '.yt'];
        $domains[] = strtolower(str_replace($search, $replace, $domain));
        sort($domains);
    }
    $d->close();
    return $domains;
}